<?php

namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use App\Exception;
use App\GeneralErrorException;
use App\NetworkException;

class WebService {
    
    const FAILOVER_EMAIL = "delivery@teknisa.com";

    private $connection;
    private $integrationService;
    private $consumerService;
    private $util;
    private $branchService;
    private $dispatchService;

    private $basePath;
    private $ordersPath;
    private $processedPath;
    private $errPath;
    private $networkErrPath;
    
    private $mailList;
    private $serverName;
    
    private $configuration;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Service\IntegrationService $integrationService,
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Service\DispatchService $dispatchService,
                                 \App\Util\Util $util,
                                 \App\Service\BranchService $branchService) {

        $this->connection          = $connection;
        $this->integrationService  = $integrationService;
        $this->consumerService     = $consumerService;
        $this->dispatchService     = $dispatchService;
        $this->util                = $util;
        $this->branchService       = $branchService;
    }

    private $errorArr = array(
        "status" => 5,
        "error"  => array(
            "type"          => "pedepronto.order.order-invalid",
            "message"       => "Há alguma inconsistência com as informações do pedido",
            "productErrors" => array(
                "id"    => "",
                "error" => array(
                    "type" => ""
                )
            )
        )
    );

    public function runWebService(Request $request, Response $response)
    {
        try {

            $req = $request->getParsedBody();
            if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") 
                throw new \Exception("Auth inválido.", 666);

            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);

            $clientFolder = $this->connection->getParams();
            $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
            
            $this->basePath          = $config['logPath'] . "/$clientFolder";
            $this->ordersPath        = $this->basePath . "/orders";
            $this->processedPath     = $this->basePath . "/processed";
            $this->errPath           = $this->basePath . "/error";
            $this->networkErrPath    = $this->basePath . "/networkError";
            $this->serverName        = $config['server'];
            
            $this->configuration = $this->integrationService->getConfiguration();
            $redirectStore       = (isset($this->configuration['IDUTLVNDPDRPALC']) && ($this->configuration['IDUTLVNDPDRPALC'] == "N"));
            $trackingEnabled     = (isset($this->configuration['IDRASTREIAENTREGA']) && ($this->configuration['IDRASTREIAENTREGA'] == "S"));
            
            if(empty($this->configuration)) {
                $response->withStatus(200);
                return 0;
            }
            
            if(empty($this->configuration['IDTOKENINTGERAL'])) {
                throw new Exception\GeneralErrorException("Token geral não encontrado.");
            }
            
            if($this->configuration['IDHABINTE'] == 'N') {
                $response->withStatus(200);
                return 0;
            }
            
            $this->mailList  = $this->util->getMailList();
            $orderData       = json_decode($this->util->executeWebService("/order", "GET", $this->configuration['IDTOKENINTGERAL']), true);

            if(isset($orderData["error"])){
                throw new Exception\GeneralErrorException("
                    Ocorreu um erro durante a autenticação com a Pede Pronto. Este erro desabilita a integração.
                    Entre em contato com a Pede Pronto e inforque o token de autenticação da integração foi recusado.
                    Após obter o token correto, faça a correção no Retail e habilite a integração novamente.
                    Em caso de dúvidas, entre em contato com a equipe de suporte Teknisa.");
            }
            
            if(empty($orderData) || (!isset($orderData["data"])) || (!count($orderData["data"]))) {
                $this->dispatchPendingOrders();
                $this->processCancelledOrders();
                $this->util->sanitizeCancelledOrderList();
                $this->itsAlive();
                $response->withStatus(200);
                return 0;
            }
            
            foreach ($orderData["data"] as $order) {
                $rawOrder = $order;
                
                $errorMessage = "";
                                
	            $this->connection->beginTransaction();
        
                try {
                    if(empty($order["pos_reference"]))
                        throw new Exception\IntegrationException("O pedido {$order['display_code']} não possui o campo pos_reference.");
                    
                    if(empty($order["id"]))
                        throw new Exception\IntegrationException("O pedido {$order['display_code']} não possui o campo id.");
                        
                    $storeInfo = $order["pos_reference"];

                    $order['branch'] = $this->branchService->fetchBranch($storeInfo);

                    $this->integrationService->validateCashierSinc($order['branch']);

                    $fileName = (isset($order['id']) ? ("{$order['branch']}-{$order['id']}-" . date("c")) : ("INDEFINIDO-" . date("H-i-s"))) . ".json"; 
                    
                    $this->util->writeLog('order', $fileName, $rawOrder);

                    $comd = $this->processOrder($order, $redirectStore);

                    $this->integrationService->validateOrder($comd['CDFILIAL'], $comd['NRVENDAREST'], $comd['NRCOMANDAEXT']);
                    
                    $this->connection->commit();

                    $this->util->writeLog('order_processed', $fileName, $rawOrder);
                    
                    $this->util->executeWebService("/order/{$order['id']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], ["status" => 4]);

                    if($trackingEnabled && $config['useOrderNotificationSystem']) {
                        $orderType = !empty($order['order_type']) ? $order['order_type'] : "";
                        $this->dispatchService->registerPendingDispatchOrder($comd["CDFILIAL"], $comd['NRCOMANDA'], $comd['NRCOMANDAEXT'], $order["pos_reference"], $orderType);
                    }
                    
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$order['id']} filial {$comd['CDFILIAL']} integrado.\n");
                    
                } catch (Exception\NetworkException $e) {
                    $this->connection->rollback();
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");

                } catch (Exception\IntegrationException $e) {
                    $this->connection->rollback();
                    if(!empty($order['id'])){
                        $this->util->executeWebService("/order/{$order['id']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], $this->errorArr);
                    
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: " . $e->getMessage() . "\n");
                    
                        if(!isset($order['branch'])) {
                            $order['branch'] = 'undefined';
                        }
                        
                        $dateLog = date("d/m/Y H:i:s");
                        $content = "[ParamsException - $dateLog]: Filial -> {$order['branch']}; codPedido -> #{$order['id']}; Error -> {$e->getMessage()}\n";
                    
                        $this->util->writeLog('order_error', $fileName, $content);
                    
                    } else {
                        continue;
                    }
                } catch (Exception\ParamsException $e) {
                    $this->connection->rollback();
                    
                    $this->util->executeWebService("/order/{$order['id']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], $this->errorArr);
                    
                    if(!isset($order['branch'])) {
                        $order['branch'] = 'undefined';
                    }
                    
                    if(!isset($fileName) || empty($fileName)){
                        $fileName = "INDEFINIDO-" . date("H-i-s") . ".json";
                    } 
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[ParamsException - $dateLog]: Filial -> {$order['branch']}; codPedido -> #{$order['id']}; Error -> {$e->getMessage()}\n";
                    
                    $this->util->writeLog('order_error', $fileName, $content);
                    
                    $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                    $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Pede Pronto", $template, [
                        ":INTEGRACAO"    => "Pede Pronto",
                        ":NUMERO_PEDIDO" => "#".$order['branch']."-".$order['id'],
                        ":NOME_SERVIDOR" => $this->serverName,
                        ":ERRO_MSG"      => $e->getMessage(),
                        ":JSON_PEDIDO"   => json_encode($order, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                        ":GERADO_EM"     => date("d/m/Y H:i:s")
                    ]);
                    
                } catch (\Exception $e) {
                    $this->connection->rollback();
                    
                    if(!isset($order['branch'])) {
                        $order['branch'] = 'undefined';
                    }
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[ParamsException - $dateLog]: Filial -> {$order['branch']}; codPedido -> #{$order['id']}; Error -> {$e->getMessage()}\n";
                    
                    $this->util->writeLog('order_error', $fileName, $content);
                    
                    $template = file_get_contents(__DIR__ ."/../../templates/error.html");
                    $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Pede Pronto", $template, [
                        ":INTEGRACAO"    => "Pede Pronto",
                        ":NUMERO_PEDIDO" => "#".$order['branch']."-".$order['id'],
                        ":NOME_SERVIDOR" => $this->serverName,
                        ":ERRO_MSG"      => $e->getMessage(),
                        ":JSON_PEDIDO"   => json_encode($order, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                        ":GERADO_EM"     => date("d/m/Y H:i:s")
                    ]);
                }
            }
            
            $this->processCancelledOrders();
            $this->util->sanitizeCancelledOrderList();
            
        } catch (Exception\NetworkException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");
            
        } catch (Exception\GeneralErrorException $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro Geral (GeneralErrorException): {$e->getMessage()}\n");
            $this->integrationService->disableIntegration();
            
            $template = file_get_contents(__DIR__ ."/../../templates/general-error.html");
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Pede Pronto", $template, [
                ":INTEGRACAO"    => "Pede Pronto",
                ":NOME_CLIENTE"  => $this->getClientName(),
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
            
        } catch (\Exception $e) {
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro Desconhecido: {$e->getMessage()}\n");
            throw new \Exception($e->getMessage());
        }    

        $response->withStatus(200);
        return 0;
    }
    
    public function processOrder ($order, $redirectStore)
    {
        //Lógica para recuperar o bairro
        if(isset($order['address']['complement']) && strstr($order['address']['complement'], 'Bairro: ')){
	        $order['address']['neighborhood'] = substr($order['address']['complement'], 8, (stripos($order['address']['complement'], ' |') -8));
	        $order['address']['complement']   = substr($order['address']['complement'], (stripos($order['address']['complement'], '|') +2));
        } else {
            $order['address']['neighborhood'] = "";
        }
        
        $deliveryRules = $this->integrationService->deliveryRules($order['branch']);
        $generalParams = $this->integrationService->generalParameters($order['branch'], $redirectStore, $deliveryRules['merchantConfiguration']);

        $generalParams["CPF"]          = "";
        $generalParams["NRCONTATOSAC"] = "";
        
        if(isset($order["customer_document"])) {
            $order["customer_document"] = preg_replace("/[^0-9]/", "", $order["customer_document"]);
            $generalParams["CPF"] = $order["customer_document"];
        }
        
        if(!empty($deliveryRules["merchantConfiguration"]["NRCONTATOSAC"])) {
            $generalParams["NRCONTATOSAC"] = $deliveryRules["merchantConfiguration"]["NRCONTATOSAC"];
        }

        $consumer = $this->consumerService->processConsumer($generalParams, $order);

        if(empty($consumer)) {
            throw new Exception\ParamsException("Não foi possível validar o consumidor do pedido.");
        }
        
        return $this->integrationService->openOrderBill($generalParams, $order, $consumer, $deliveryRules);
    }
    
    public function welcome(Request $request, Response $response)
    {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
        ]);
    }
    
    private function getVersionInfo()
    {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    public function debugOrder(Request $request, Response $response)
    {
        $req = $request->getParsedBody();

        if ($req["auth"] != "81743A8E8D41AF82A4807097CDF4BEC2DDDB9E6334BECCFED4D46FC783A8CA2E") 
            throw new \Exception("Auth inválido.", 666);
            
        $this->configuration = $this->integrationService->getConfiguration();

        $redirectStore       = (isset($this->configuration['IDUTLVNDPDRPALC']) && ($this->configuration['IDUTLVNDPDRPALC'] == "N"));
        $trackingEnabled     = (isset($this->configuration['IDRASTREIAENTREGA']) && ($this->configuration['IDRASTREIAENTREGA'] == "S"));
        
        $configsFile         = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config              = json_decode($configsFile, true);
            
        if(empty($this->configuration)) {
            $response->withStatus(200);
            return 0;
        }
        
        if(empty($this->configuration['IDTOKENINTGERAL'])) {
            throw new Exception\GeneralErrorException("Token geral não encontrado.");
        }
        
        $this->mailList  = $this->util->getMailList();
        
        $errorMessage = "";

        try{
            $this->connection->beginTransaction();
            
            $order = $req["data"];
            
            if(empty($order["id"]))
                throw new Exception\IntegrationException("O pedido {$order['display_code']} não possui o campo id.");
            
            //Token da Filial Teknisa...
            $order['branch']        = "0001";
            //$order["pos_reference"] = "16242555";
            
            $storeInfo = $order["pos_reference"];

            $order['branch'] = $this->branchService->fetchBranch($storeInfo);
            
            $fileName = (isset($order['id']) ? ("{$order['branch']}-{$order['id']}-" . date("c")) : ("INDEFINIDO-" . date("H-i-s"))) . ".json"; 
            
            $this->util->writeLog('order', $fileName, $order);
            
            $comd = $this->processOrder($order, $redirectStore);

            $this->integrationService->validateOrder($comd['CDFILIAL'], $comd['NRVENDAREST'], $comd['NRCOMANDAEXT']);

            $this->connection->commit();
            
            if($trackingEnabled && $config['useOrderNotificationSystem']) {
                $orderType = !empty($order['order_type']) ? $order['order_type'] : "";
                
                $this->dispatchService->registerPendingDispatchOrder($comd["CDFILIAL"], $comd['NRCOMANDA'], $comd['NRCOMANDAEXT'], $order["pos_reference"], $orderType);
            }
            
            $this->util->writeLog('order_processed', $fileName, $order);
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$order['id']} filial {$comd['CDFILIAL']} integrado.\n");

        } catch (Exception\NetworkException $e) {
            $this->connection->rollback();
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");

        } catch (Exception\IntegrationException $e) {
            $this->connection->rollback();
            
            
            if(!empty($order['id'])){
                $this->util->executeWebService("/order/{$order['id']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], $this->errorArr);
            
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: " . $e->getMessage() . "\n");
            
                if(!isset($order['branch'])) {
                    $order['branch'] = 'undefined';
                }
                
                $dateLog = date("d/m/Y H:i:s");
                $content = "[ParamsException - $dateLog]: Filial -> {$order['branch']}; codPedido -> #{$order['id']}; Error -> {$e->getMessage()}\n";
            
                $this->util->writeLog('order_error', $fileName, $content);
            
            } else {
                return $response->withJson([
                    'success' => false,
                    'error'   => "Campo id ausente"
                ]);
            }
        } catch (Exception\ParamsException $e) {
            
            $errorMessage = $e->getMessage();
            $this->connection->rollback();
            
            $this->util->executeWebService("/order/{$order['id']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], $this->errorArr);
            
            if(!isset($order['branch'])) {
                $order['branch'] = 'undefined';
            }
            
            if(!isset($fileName) || empty($fileName)){
                $fileName = "INDEFINIDO-" . date("H-i-s") . ".json";
            } 
            
            $dateLog = date("d/m/Y H:i:s");
            $content = "[ParamsException - $dateLog]: Filial -> {$order['branch']}; codPedido -> #{$order['id']}; Error -> {$e->getMessage()}\n";
            
            $this->util->writeLog('order_error', $fileName, $content);
            
            $template = file_get_contents(__DIR__ ."/../../templates/error.html");
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Pede Pronto", $template, [
                ":INTEGRACAO"    => "Pede Pronto",
                ":NUMERO_PEDIDO" => "#".$order['branch']."-".$order['id'],
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":JSON_PEDIDO"   => json_encode($order, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
            
        } catch (\Exception $e) {
            
            $errorMessage = $e->getMessage();
            $this->connection->rollback();
            
            if(!isset($order['branch'])) {
                $order['branch'] = 'undefined';
            }
            
            $dateLog = date("d/m/Y H:i:s");
            $content = "[ParamsException - $dateLog]: Filial -> {$order['branch']}; codPedido -> #{$order['id']}; Error -> {$e->getMessage()}\n";
            
            $this->util->writeLog('order_error', $fileName, $content);
            
            $template = file_get_contents(__DIR__ ."/../../templates/error.html");
            $this->util->sendMail((!empty($this->mailList) ? $this->mailList : [self::FAILOVER_EMAIL]), [], "Integração Teknisa x Pede Pronto", $template, [
                ":INTEGRACAO"    => "Pede Pronto",
                ":NUMERO_PEDIDO" => "#".$order['branch']."-".$order['id'],
                ":NOME_SERVIDOR" => $this->serverName,
                ":ERRO_MSG"      => $e->getMessage(),
                ":JSON_PEDIDO"   => json_encode($order, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE),
                ":GERADO_EM"     => date("d/m/Y H:i:s")
            ]);
        }
        
        if($errorMessage) {
            return $response->withJson([
            'success' => false,
            'errorMessage' => $errorMessage
        ]);
        }
        
        return $response->withJson([
            'success' => true,
            'orderId' => $req['data']['id']
        ]);
    }
    
    private function getClientName()
    {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    public function itsAlive()
    {
        $executionLogFilePath  = $this->util->getExecutionLog();
        $executionLogTimeStamp = filemtime($executionLogFilePath);
        if((time() - $executionLogTimeStamp) >= 3600){
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Integração] Verificação de integridade.\n");
        }
    }
    
    /* 
     * Controle de status do delivery direto 
    */
    private function dispatchPendingOrders()
    {
        $pendingOrdersFile = $this->util->getPendingOrdersFile();
        
        $this->filterDispatchFile($pendingOrdersFile);
        $this->processPendingDispatchOrders($pendingOrdersFile);
        $this->processPendingDeliveredOrders($pendingOrdersFile);
        
        return 0;
    }
    
    private function filterDispatchFile($pendingOrdersFile)
    {
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData)) {
            return 0;
        }

        $activeList = [];
        foreach ($pendingDispatchData as $key => $currentOrder) {
            
            $diff = (time() - $currentOrder['createdAt']);
            if($diff > 28800) { // 8h
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Pedido Delivery] #" . $currentOrder['nrComandaExt'] . " expirou (não foi expedido dentro do tempo limite de 8h)." . PHP_EOL);
                continue;
            }

            $activeList[$key] = $currentOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
    
    private function processPendingDispatchOrders($pendingOrdersFile)
    {
        $pendingDispatchFile = file_get_contents($pendingOrdersFile);   
        $pendingDispatchData = json_decode($pendingDispatchFile, true);

        if(empty($pendingDispatchData))
            return 0;

        $cdFilialArr     = [];
        $nrComandaArr    = [];
        $nrComandaExtArr = [];
        
        foreach($pendingDispatchData as $pendingDispatchOrder) {
            
            if(!empty($pendingDispatchOrder['dispatchedAt']) || $pendingDispatchOrder['delivered']) {
                continue;
            }
            
            if(!isset($pendingDispatchOrder['cdFilial']) || empty($pendingDispatchOrder['cdFilial'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComanda']) || empty($pendingDispatchOrder['nrComanda'])) {
                continue;                
            }            
            
            if(!isset($pendingDispatchOrder['nrComandaExt']) || empty($pendingDispatchOrder['nrComandaExt'])) {
                continue;                
            }            

            array_push($cdFilialArr,     $pendingDispatchOrder['cdFilial']);
            array_push($nrComandaArr,    $pendingDispatchOrder['nrComanda']);
            array_push($nrComandaExtArr, $pendingDispatchOrder['nrComandaExt']);
        }
        
        if(empty($cdFilialArr)) 
            return 0;
        
        if(empty($nrComandaArr)) 
            return 0;
        
        if(empty($nrComandaExtArr)) 
            return 0;
        
        $orders = $this->dispatchService->getPendingDispatchOrders($cdFilialArr, $nrComandaArr, $nrComandaExtArr);

        if(empty($orders)){
            return 0;
        }
        
        foreach($orders as $order) {
            $key = "{$order['CDFILIAL']}|{$order['NRCOMANDA']}|{$order['NRCOMANDAEXT']}";
     
            if($order['IDSTCOMANDA'] == '4') {
                if(isset($pendingDispatchData[$key])) {
                    $this->util->executeWebService("/order/{$order['NRCOMANDAEXT']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], ["status" => 4]);
                    $pendingDispatchData[$key]['canceled'] = true;
                    
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " foi removido da fila de controle de entregador (motivo: pedido cancelado)." . PHP_EOL);
                }
            }
            
            if($order['IDSTCOMANDA'] == '5' || $order['IDSTCOMANDA'] == 'X') {
                if(isset($pendingDispatchData[$key])) {
                    $orderType = $pendingDispatchData[$key]['orderType'];

                    if ($orderType == "5") {
                        $this->util->executeWebService("/order/{$order['NRCOMANDAEXT']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], ["status" => 7]);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Pedido Delivery] #" . $pendingDispatchData[$key]['nrComandaExt'] . " saiu para entrega." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();   
                        
                    } else if ($orderType == "1") {
                        $this->util->executeWebService("/order/{$order['NRCOMANDAEXT']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], ["status" => 6]);
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Pedido Retirada] #" . $pendingDispatchData[$key]['nrComandaExt'] . " está pronto para retirada." . PHP_EOL);
                        $pendingDispatchData[$key]['dispatchedAt'] = time();
                        
                    } else {
                        $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . " O pedido #" . $pendingDispatchData[$key]['nrComandaExt'] . " foi removido da fila de entrega." . PHP_EOL);
                        $pendingDispatchData[$key]['canceled'] = true;
                    }
                }
            }
        }
        
        $activeList = [];
        
        foreach ($pendingDispatchData as $key => $currentOrder) {
            if($currentOrder['canceled']) {
                continue;
            }
            $activeList[$key] = $currentOrder;
        }
        
        file_put_contents($pendingOrdersFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
    
    private function processPendingDeliveredOrders($pendingOrdersFile)
    {
        $pendingDeliveredFile = file_get_contents($pendingOrdersFile);   
        $pendingDeliveredData = json_decode($pendingDeliveredFile, true);
        
        if(empty($pendingDeliveredData))
            return 0;
            
        
        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            
            if(empty($pendingDeliveredOrder['dispatchedAt'])) {
                continue;
            }

            $diff = (time() - $pendingDeliveredOrder['dispatchedAt']);
            if($diff >= 3600) {//3600
                
                $this->util->executeWebService("/order/{$pendingDeliveredOrder['nrComandaExt']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], ["status" => 8]);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s") . ": [Pedido Delivery] #" . $pendingDeliveredOrder["nrComandaExt"] . " foi entregue." . PHP_EOL);
                
                $pendingDeliveredData[$key]['delivered'] = true;
            }
        }
    
        $newPendingList = [];
        
        foreach($pendingDeliveredData as $key => $pendingDeliveredOrder) {
            if($pendingDeliveredOrder['delivered']) {
                continue;
            }
            $newPendingList[$key] = $pendingDeliveredOrder;
        }

        file_put_contents($pendingOrdersFile, json_encode($newPendingList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
 
    }
    
    private function processCancelledOrders()
    {
        $timeRange              = date('Y-m-d\TH:i:s', strtotime('-5 minutes'));
        $newCancelledOrders     = json_decode($this->util->executeWebService("/order?status=0&since=" . $timeRange, "GET", $this->configuration['IDTOKENINTGERAL']), true);
        $newCancelledOrdersData = $newCancelledOrders['data'];

        $cancelledOrderFile      = $this->util->getCancelledOrdersList();
        $cancelledOrderData      = file_get_contents($cancelledOrderFile);
        $cancelledOrderDataArray = json_decode($cancelledOrderData, true);

        if(isset($newCancelledOrdersData) && !empty($newCancelledOrdersData)){
            foreach($newCancelledOrdersData as $newCancelledOrder) {

                try {
                    if(empty($newCancelledOrder["pos_reference"]))
                        throw new Exception\IntegrationException("O pedido {$newCancelledOrder['display_code']} não possui o campo pos_reference.");

                    $storeInfo = $this->branchService->fetchBranch($newCancelledOrder['pos_reference']);
                    $orderData = $this->integrationService->getOrderData($newCancelledOrder['id'], $storeInfo);

                    $key = "{$orderData['CDFILIAL']}|{$orderData['NRCOMANDAEXT']}";

                    if($orderData['IDSTCOMANDA'] == '1' || $orderData['IDSTCOMANDA'] == '2' || $orderData['IDSTCOMANDA'] == '3'){
                        $isPossible = $this->integrationService->checkIfIsPossibleToCancel($orderData['NRCOMANDAEXT']);
                        if(empty($isPossible)){
                            $this->integrationService->cancelOrder($cancelledOrderFile, $orderData);
                            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderData['NRCOMANDAEXT']} cancelado pela Pede Pronto.\n");
                        } else {
                            $this->rejectCancelOrder($orderData);
                        }
                    } else {
                        $this->rejectCancelOrder($orderData);
                    }
                     
                } catch (Exception\NetworkException $e) {
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro de Rede (NetworkException): {$e->getMessage()}\n");
            
                } catch (Exception\IntegrationException $e) {
                    $this->connection->rollback();
                    $this->util->executeWebService("/order/{$orderData['NRCOMANDAEXT']}", "PATCH", $this->configuration['IDTOKENINTGERAL'], $this->errorArr);
                    
                    $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Error: " . $e->getMessage() . "\n");
                    
                    if(!isset($orderData['CDFILIAL'])) {
                        $orderData['CDFILIAL'] = 'undefined';
                    }
                    
                    $dateLog = date("d/m/Y H:i:s");
                    $content = "[ParamsException - $dateLog]: Filial -> {$orderData['CDFILIAL']}; codPedido -> #{$orderData['NRCOMANDAEXT']}; Error -> {$e->getMessage()}\n";
                    
                    $this->util->writeLog('order_error', $fileName, $content);
                }
            }
        }
    }
    
    private function rejectCancelOrder($orderData)
    {
        $cancelledOrderFile      = $this->util->getCancelledOrdersList();
        $cancelledOrderData      = file_get_contents($cancelledOrderFile);
        $cancelledOrderDataArray = json_decode($cancelledOrderData, true);
        
        $key = "{$orderData['CDFILIAL']}|{$orderData['NRCOMANDAEXT']}";
        !$inArray = false;
        
        if(isset($cancelledOrderDataArray) && !empty($cancelledOrderDataArray)) {
            
            foreach($cancelledOrderDataArray as $cancelledOrderInArray) {
                if (strval($orderData['NRCOMANDAEXT']) == $cancelledOrderInArray['NRCOMANDAEXT']) {
                    $inArray = true;
                } 
            }
        }
        
        if(!$inArray) {
            switch($orderData['IDSTCOMANDA']){
                case '4':
                    $desc = 'Cancelado';
                    break;
                case 'P':
                    $desc = 'Impresso';
                    break;
                case '5':
                    $desc = 'Expedido';
                    break;
                case 'X':
                    $desc = 'Finalizado';
                    break;
                default:
                    $desc = 'Outros';
                    break;
            }
            
            $orderToCancel = [
                'CDFILIAL'     => $orderData['CDFILIAL'],
                'NRCOMANDAEXT' => $orderData['NRCOMANDAEXT'],
                'TIMESTAMP'    => time(),
                'CANCELLED'    => false
            ];

            $cancelledOrderDataArray[$key] = $orderToCancel;
            $cancelledOrderData = json_encode($cancelledOrderDataArray, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
            file_put_contents($cancelledOrderFile, $cancelledOrderData);
            
            if($orderData['IDSTCOMANDA'] == '1' || $orderData['IDSTCOMANDA'] == '2' || $orderData['IDSTCOMANDA'] == '3') {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderData['NRCOMANDAEXT']} não pode ser cancelado pois não existe uma venda relacionada com a comanda. O cancelamento deste pedido deve ser feito manualmente.\n");
            } else {
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido #{$orderData['NRCOMANDAEXT']} não foi cancelado pois o status atual é '{$desc}' e não pode ser cancelado.\n");
            }
        }
    }
}
