<?php
namespace App\Factory;

use App\Exception;

class BranchFactory {
    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection)
    {
        $this->connection = $connection;
    }
    
    public function fetchBranch($storeInfo)
    {
        $sql = "
            SELECT CDFILIAL 
              
              FROM INTEGRACAOFILI IF 
              
              JOIN INTEGRACAO I
                ON IF.CDINTEGRACAO = I.CDINTEGRACAO
            
             WHERE I.IDINTEGRACAO  = 'PED'
               AND IF.IDTOKEN      = :CDFILIALEXT
        ";

        $params = [
            ":CDFILIALEXT"  => $storeInfo
        ];
        
        $branchInfo = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($branchInfo['CDFILIAL'])) {
            throw new Exception\ParamsException("
                Não existe uma filial associada ao Pos Reference {$storeInfo}.<br>
                Para associar uma filial a este Pos Reference, verifique a seção 'Configurando o Token das Filiais', no manual da integração.");
        }

        return $branchInfo['CDFILIAL'];
    }
    
    public function getStandardVendor ($branch)
    {
        
		$sql = "
            SELECT CDVENDEDOR
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL";
            
        $params = [
            ':CDFILIAL' => $branch    
        ];
        
        $vendor = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($vendor['CDVENDEDOR'])) 
            throw new Exception\ParamsException("
                Não existe um vendedor padrão atrelado a filial {$branch}.<br>
                Para atrelar o vendedor padrão desta filial, verifique a seção 'Parametrização do Delivery', no manual da integração.");
        
        return $vendor['CDVENDEDOR'];
    }
    
    public function getStandardStore ($branch, $vendor)
    {
        $sql = "
            SELECT C.CDLOJA 
              FROM VENDEDOR V
              
              JOIN CAIXA C
                ON C.CDCAIXA    = V.CDCAIXA
              
              JOIN LOJA L 
                ON C.CDLOJA     = L.CDLOJA
               AND C.CDFILIAL   = L.CDFILIAL
            
             WHERE V.CDVENDEDOR = :CDVENDEDOR
               AND C.CDFILIAL   = :CDFILIAL        
        ";
            
        $params = [
            ":CDFILIAL"   => $branch,
            ":CDVENDEDOR" => $vendor
        ];
        
        $store = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($store['CDLOJA'])) 
            throw new Exception\ParamsException("
                Não existe uma loja associada a filial {$branch} e vendedor padrão código {$vendor}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        return $store['CDLOJA'];
    }
    
    public function getStandardCashier ($branch, $vendor)
    {
        $sql = "
            SELECT CDCAIXA, CDOPERADOR
              FROM VENDEDOR
             WHERE CDFILIAL   = :CDFILIAL
               AND CDVENDEDOR = :CDVENDEDOR";
            
        $params = [
            ":CDFILIAL"   => $branch,
            ":CDVENDEDOR" => $vendor
        ];
        
        $cashier = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($cashier['CDCAIXA'])) 
            throw new Exception\ParamsException("
                Não existe um caixa associado a filial {$branch} e vendedor padrão código {$vendor}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        return $cashier;
    }
    
    public function getProductControlId ($branch, $store)
    {
        $sql = "
            SELECT IDCONTROPROD
              FROM LOJA
             WHERE CDFILIAL = :CDFILIAL
               AND CDLOJA   = :CDLOJA";
              
        $params = [
            ":CDFILIAL" => $branch,
            ":CDLOJA"   => $store
        ];
        
        $idControlProd = $this->connection->fetchAssoc($sql, $params);
        
        if (empty($idControlProd["IDCONTROPROD"]))
            throw new Exception\ParamsException("
                A tela de controle de produção não está parametrizada para a filial {$branch}.<br>
                Caso necessite de ajuda com esta parametrização, entre em contato com a equipe de suporte Teknisa.");
        
        return $idControlProd["IDCONTROPROD"];
    }
    
    public function getStandartClient ($branch)
    {
        $sql = "
            SELECT CDCLIENTE
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL";
            
        $params = [
            ":CDFILIAL" => $branch
        ];
        
        $stdClient = $this->connection->fetchAll($sql, $params)[0];
        
        if (empty($stdClient['CDCLIENTE'])) 
            throw new Exception\ParamsException("Não foi encontrado cliente padrão para a filial - {$branch}", 105);
        
        return $stdClient;
    }
    
    public function getStandartConsumerType ($branch)
    {
        $sql = "
            SELECT CDTIPOCONS, CDCCUSCLIE
              FROM PARAVEND
             WHERE CDFILIAL = :CDFILIAL";
            
        $params = [
            ":CDFILIAL" => $branch    
        ];
        
        $consumerT = $this->connection->fetchAll($sql, $params)[0];
        
        if (empty($consumerT['CDTIPOCONS']) || empty($consumerT['CDCCUSCLIE']))
            throw new Exception\ParamsException("Não foi encontrado tipo de consumidor padrão para a filial - {$branch}", 106);
        
        return $consumerT;
    }
    
    public function getBranchCityCode($branch)
    {
        $sql = "
            SELECT CDMUNICIPIO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL";
             
        $params = array(
            ":CDFILIAL" => $branch
        );
        
        return $this->connection->fetchAll($sql, $params)[0]["CDMUNICIPIO"];
    }
    
    public function getBranchState ($branch)
    {
        $sql = "
            SELECT SGESTADO
              FROM ENDEFILI
             WHERE CDFILIAL = :CDFILIAL
        ";
        
        $params = [
            ":CDFILIAL" => $branch
        ];
        
        $sgEstado = $this->connection->fetchAll($sql, $params)[0]['SGESTADO'];
        return $sgEstado;
    }
    
    public function getAddressInfo ($branch)
    {
        $sql = "
            SELECT CDMUNICIPIO, SGESTADO, DSENDEFILI, NMBAIRFILI, NRCEPFILI
              FROM ENDEFILI
            WHERE CDFILIAL = :CDFILIAL
              AND IDTPENDEFILI = 'P'";
        
        $params = array(
            ":CDFILIAL" => $branch    
        );
        
        return $this->connection->fetchAll($sql, $params)[0];
    }
    
    public function isPosEnabledToAutomaticOrderPrinting ($branch, $store, $cashier)
    {
        $sql = "
            SELECT IDIMPAUTCUPEXP, IDIMPAUTCUPFOS
              FROM CAIXA
             WHERE CDFILIAL = :CDFILIAL
               AND CDLOJA   = :CDLOJA
               AND CDCAIXA  = :CDCAIXA";
               
        $params = [
             ":CDFILIAL" => $branch,
             ":CDLOJA"   => $store,
             ":CDCAIXA"  => $cashier
        ]; 
        
        $isAutomaticPrint = $this->connection->fetchAssoc($sql, $params);
        if ((empty($isAutomaticPrint["IDIMPAUTCUPEXP"])) && (empty($isAutomaticPrint["IDIMPAUTCUPFOS"]))) {
            return null;
        }
            
        return $isAutomaticPrint;
    }
}
