<?php

namespace App\Util;

use App\Exception;
use App\GeneralErrorException;
use App\NetworkException;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

class Util {
    
    const PED_BASE_URL = "https://pos.onyo.com/v1/teknisa"; //produção
    //const PED_BASE_URL = "https://pos.staging.onyo.com/v1/teknisa"; //teste 
    
    
    private $connection;
    private $dbSchema = 'pt-br';
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
        $this->loadDbSchema();
    }
    
    private function getDbSchema() {
        return $this->dbSchema;
    }
    
    private function loadDbSchema() {
        global $ENVIRONMENT_FILE;

        $environmentFile = $ENVIRONMENT_FILE;
        $environment     = json_decode($environmentFile, true);
        
        if(isset($environment["dbSchema"])){
            $this->dbSchema = $environment["dbSchema"];
        }
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }

    public function executeWebService ($function, $method, $token, $params = []) {
        $header = [
            "Content-type: application/json",
            "Authorization: Bearer " .$token
        ];
        
        $url = self::PED_BASE_URL . $function;
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        
        if(!empty($params)) {
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        }

        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        $response = curl_exec($ch);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        if (!empty(curl_error($ch)) || stripos($response, "<html>")) {
            $fileName = date("H-i-s") . ".txt";

            $this->writeLog('network', $fileName, $this->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $header,
                'url'       => $url,
                'method'    => $method,
                'params'    => $params,
                'httpCode'  => $httpCode,
                'error'     => curl_error($ch),
                'response'  => $response
            ]));

            throw new Exception\NetworkException("Request error at {$url}");
        }
        
        curl_close($ch);
        return $response;
    }

    private function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../Templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }    

    public function writeLog($type, $fileName, $content) {
        
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";

        $basePath     = $config['logPath'] . "/$clientFolder";
        
        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content);
        }
        
        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
            $currentFile = "$folder" . date("d") . ".txt";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }

        if($type == 'events') {
            $folder      = $basePath . "/events/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

	        file_put_contents($currentFile, $content, FILE_APPEND);
        }

        if($type == 'cancellation') {
            $folder      = $basePath . "/cancellation/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

	        file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }
    }

    public function sendMail($recipient, $cc, $subject, $body, $params) 
    {
        
        return 0;
        try {
            $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config     = json_decode($configFile, true);
        
            $mail = new PHPMailer(true);

            $mail->isSMTP();                            
            $mail->Host       = $config["host"];          
            $mail->SMTPAuth   = true;                   
            $mail->Username   = $config["username"];
            $mail->Password   = $this->decrypt($config["password"]); 
            $mail->SMTPSecure = $config["SMTPSecure"];    
            $mail->Port       = $config["port"];      
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";
            
            $mail->setFrom($config["from"], "Teknisa Retail"); 
            
            if(empty($cc)) {
                //$cc = ["delivery@teknisa.com"];
                $cc = ["suporte.odhen@teknisa.com"];
            }

            foreach ($recipient as $r) { $mail->addAddress($r); }
            foreach ($cc as $c) { $mail->addCC($c); }
            
            $mail->isHTML(true);                                  
            $mail->Subject = $subject;
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();
        } catch (Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }
    
    public function tofloat($num)
    {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
        
        $tmp2 = str_replace(".", ",", $tmp);

        if($this->getDbSchema() == 'en-us') {
            $tmp2 = str_replace(",", ".", $tmp2);
        }
        
        return $tmp2; 
    }
    
    //Pede Pronto
    private function decrypt($string)
    {
        $encrypt_method = "AES-256-CBC";
        $secret_key     = 'ZEEDHI';
        $secret_iv      = 'ZEEDHI IV';

        $key = hash('sha256', $secret_key);
        $iv  = substr(hash('sha256', $secret_iv), 0, 16);
        return openssl_decrypt(base64_decode($string), $encrypt_method, $key, 0, $iv);
    }
    
    public function getMailList()
    {
        $list = [];

        $sql = "
            SELECT DSEMAILTEC 
              FROM LISTAEMAILTEC
             WHERE CDPROJETO = '005'
        ";
        
        $result = $this->connection->fetchAll($sql);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("
                Não existe um e-mail cadastrado para receber os erros da integração.<br>
                Para cadastrar um e-mail, verifique a seção 'Parametrização da Integração', no manual da integração.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("
                Não existe um e-mail válido cadastrado para receber os erros da integração.<br>
                Verifique a seção 'Parametrização da Integração', no manual da integração.");
        }

        return $list;
    }
    
    public function formatNrSeqProdCom($seq)
    {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }
    
    public function getExecutionLog()
    {
        $configFile = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config = json_decode($configFile, true);
        
        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";
        $folder       = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
        $currentFile  = "$folder" . date("d") . ".txt";
        
        $content = "";
        
        if(!file_exists($folder)) {
            mkdir($folder, 0775, true);
            
        }
        if(!file_exists($currentFile)) {
            $content = date("d/m/Y H:i:s") . ": [Integração] Verificação de integridade.\n";
        }
        file_put_contents($currentFile, $content, FILE_APPEND);
        
        return $currentFile;
    }
    
    public function getPendingOrdersFile()
    {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";

        $folder = $basePath."/dispatch";

        $fileName = "/pendingOrders.json";
        $currentFile = $folder . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }
        
        if(!file_exists($currentFile)){
            file_put_contents($currentFile, json_encode([]));
        }

        return $currentFile;
    }
    
    public function getCancelledOrdersList()
    {
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        
        $basePath     = $config['logPath'] . "/$clientFolder";

        $folder = $basePath."/cancelled";

        $fileName = "/cancelledOrders.json";
        $currentFile = $folder . $fileName;
        
        if(!file_exists($folder)){
            mkdir($folder, 0775, true);
            file_put_contents($currentFile, json_encode([]));
        }
        
        if(!file_exists($currentFile)){
            file_put_contents($currentFile, json_encode([]));
        }

        return $currentFile;
    }
    
    public function sanitizeCancelledOrderList()
    {
        $cancelledOrderFile      = $this->getCancelledOrdersList();
        $cancelledOrderData      = file_get_contents($cancelledOrderFile);
        $cancelledOrderDataArray = json_decode($cancelledOrderData, true);

        if(empty($cancelledOrderDataArray)) {
            return 0;
        }
        
        $newCancelledOrderData = $cancelledOrderDataArray;
        
        $activeList = [];
        foreach ($cancelledOrderDataArray as $key => $currentOrder) {
            
            $diff = (time() - $currentOrder['TIMESTAMP']);
            if($diff > 604800) { // 7d
                continue;
            }

            $activeList[$key] = $currentOrder;
        }

        file_put_contents($cancelledOrderFile, json_encode($activeList, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
    }
}